#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2024-07-06
# File  : vsphere_topo/rulesets/packages.py

from collections.abc import Mapping, Sequence

from cmk.rulesets.v1 import Label, Title
from cmk.rulesets.v1.form_specs import (
    CascadingSingleChoice,
    CascadingSingleChoiceElement,
    DefaultValue,
    DictElement,
    Dictionary,
    FixedValue,
    SingleChoice,
    SingleChoiceElement,
    String,
)

from cmk.rulesets.v1.form_specs.validators import LengthInRange
from cmk.rulesets.v1.rule_specs import CheckParameters, HostCondition, Topic
from cmk_addons.plugins.vsphere_topo.lib.utils import (
    EMBLEM_CLUSTER,
    EMBLEM_DATA_CENTER,
    EMBLEM_DATA_STORE,
    PARAM_ADD_DUMMY_TOPOLOGIES,
    PARAM_CHANGE_CASE,
    PARAM_CLUSTER,
    PARAM_CUSTOM_EMBLEM,
    PARAM_DATA_CENTER,
    PARAM_DATA_STORE,
    PARAM_DATA_STORE_AS_SERVICE,
    PARAM_DEFAULT_EMBLEM,
    PARAM_DONT_ADD_VC_AS_VM,
    PARAM_HOST_SYSTEMS,
    PARAM_KEEP_DOMAIN,
    PARAM_LOWER,
    PARAM_MAKE_DEFAULT,
    PARAM_NO_CHANGE,
    PARAM_NO_EMBLEM,
    PARAM_PREFIX,
    PARAM_UPPER,
    PARAM_VM_NAMES,
    RULE_SET_NAME_VSPHERE_TOPO,
)

adjust_names_elements: Mapping[str, DictElement] = {
    PARAM_KEEP_DOMAIN: DictElement(
        parameter_form=FixedValue(
            title=Title('Keep domain name'),
            label=Label('The domain name will not be cut'),
            value=True,
        )),
    PARAM_CHANGE_CASE: DictElement(
        parameter_form=SingleChoice(
            title=Title('Change case'),
            elements=[
                SingleChoiceElement(name=PARAM_UPPER, title=Title('All upper case')),
                SingleChoiceElement(name=PARAM_LOWER, title=Title('All lower case')),
                SingleChoiceElement(name=PARAM_NO_CHANGE, title=Title('Don\'t change case'))
            ],
            prefill=DefaultValue(PARAM_NO_CHANGE)
        )),
    PARAM_PREFIX: DictElement(
        parameter_form=String(
            title=Title('Name prefix'),
            custom_validate=(LengthInRange(min_value=1),)
        )),
}


def get_emblem_element(default_emblem) -> Sequence[CascadingSingleChoiceElement]:
    return [
        CascadingSingleChoiceElement(
            name=PARAM_NO_EMBLEM,
            title=Title('No custom emblem'),
            parameter_form=FixedValue(
                value=True,
                label=Label('No custom emblem will be used')
            )),
        CascadingSingleChoiceElement(
            name=PARAM_DEFAULT_EMBLEM,
            title=Title('Use default emblem'),
            parameter_form=FixedValue(
                value=True,
                label=Label(f'Emblem "{default_emblem}" will be used')
            )),
        CascadingSingleChoiceElement(
            name=PARAM_CUSTOM_EMBLEM,
            title=Title('Use custom emblem'),
            parameter_form=String(
                custom_validate=(LengthInRange(min_value=1),),
                prefill=DefaultValue(default_emblem),
            ))
    ]


def _parameter_form() -> Dictionary:
    return Dictionary(
        elements={
            PARAM_HOST_SYSTEMS: DictElement(
                parameter_form=Dictionary(
                    title=Title('Adjust ESXi host names'),
                    elements=adjust_names_elements
                )),
            PARAM_VM_NAMES: DictElement(
                parameter_form=Dictionary(
                    title=Title('Adjust VM names'),
                    elements=adjust_names_elements
                )),
            PARAM_CLUSTER: DictElement(
                parameter_form=CascadingSingleChoice(
                    title=Title('Cluster emblem'),
                    elements=get_emblem_element(EMBLEM_CLUSTER),
                    prefill=DefaultValue(PARAM_DEFAULT_EMBLEM)
                )),
            PARAM_DATA_CENTER: DictElement(
                parameter_form=CascadingSingleChoice(
                    title=Title('Datacenter emblem'),
                    elements=get_emblem_element(EMBLEM_DATA_CENTER),
                    prefill=DefaultValue(PARAM_DEFAULT_EMBLEM)
                )),
            PARAM_DATA_STORE: DictElement(
                parameter_form=CascadingSingleChoice(
                    title=Title('Datastore emblem'),
                    elements=get_emblem_element(EMBLEM_DATA_STORE),
                    prefill=DefaultValue(PARAM_DEFAULT_EMBLEM)
                )),
            PARAM_DATA_STORE_AS_SERVICE: DictElement(
                parameter_form=FixedValue(
                    title=Title('Add data store service'),
                    label=Label('enabled'),
                    value=True
                )),
            PARAM_MAKE_DEFAULT: DictElement(
                parameter_form=FixedValue(
                    title=Title('Make default'),
                    label=Label('This will be the default topology'),
                    value=True
                )),
            PARAM_DONT_ADD_VC_AS_VM: DictElement(
                parameter_form=FixedValue(
                    title=Title('Don\'t add vCenter as VM'),
                    label=Label('The vCenter will not be added as VM'),
                    value=True
                )),
            PARAM_ADD_DUMMY_TOPOLOGIES: DictElement(
                parameter_form=FixedValue(
                    title=Title('Add dummy topologies'),
                    label=Label('Adds empty CDP, LLDP, L3v4 and STATIC topology'),
                    value=True
                )),
        }
    )


rule_spec_vsphere_topo = CheckParameters(
    name=RULE_SET_NAME_VSPHERE_TOPO,
    topic=Topic.APPLICATIONS,
    parameter_form=_parameter_form,
    title=Title('vSphere Topology'),
    condition=HostCondition(),
)
